"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createAceCompatAutocompleter = exports.createCompletionResultForIdPrefix = exports.ID_REGEX = exports.mapMongoDBCompletionToCodemirrorCompletion = void 0;
const autocomplete_1 = require("@codemirror/autocomplete");
const autocompleter_1 = require("../autocompleter");
const utils_1 = require("./utils");
function mapMongoDBCompletionToCodemirrorCompletion(completion, escape = 'invalid') {
    const cmCompletion = {
        label: completion.value,
        apply: escape === 'never'
            ? completion.value
            : (0, autocompleter_1.wrapField)(completion.value, escape === 'always'),
        detail: completion.meta?.startsWith('field') ? 'field' : completion.meta,
        type: completion.meta?.startsWith('field') ? 'field' : 'method',
        info() {
            if (!completion.description) {
                return null;
            }
            const infoNode = document.createElement('div');
            infoNode.classList.add('completion-info');
            infoNode.addEventListener('mousedown', (evt) => {
                // If we are clicking a link inside the info block, we have to prevent
                // default browser behavior that will remove the focus from the editor
                // and cause the autocompleter to dissapear before browser handles the
                // actual click. This is very similar to how codemirror handles clicks
                // on the list items
                // @see {@link https://github.com/codemirror/autocomplete/blob/82480a7d51d60ad933808e42f6189d841a5a6bc8/src/tooltip.ts#L96-L97}
                if (evt.target.nodeName === 'A') {
                    evt.preventDefault();
                }
            });
            infoNode.innerHTML = completion.description;
            return infoNode;
        },
    };
    if (completion.snippet) {
        return (0, autocomplete_1.snippetCompletion)(completion.snippet, cmCompletion);
    }
    return cmCompletion;
}
exports.mapMongoDBCompletionToCodemirrorCompletion = mapMongoDBCompletionToCodemirrorCompletion;
/**
 * Ace autocompleter "valid" identifier regex
 */
exports.ID_REGEX = /[a-zA-Z_0-9$\-\u00A2-\u2000\u2070-\uFFFF]+/;
function createCompletionResultForIdPrefix({ prefix, completions, escape, }) {
    return {
        from: prefix.from,
        to: prefix.to,
        options: completions.map((completion) => {
            return mapMongoDBCompletionToCodemirrorCompletion(completion, escape);
        }),
        validFor: exports.ID_REGEX,
    };
}
exports.createCompletionResultForIdPrefix = createCompletionResultForIdPrefix;
/**
 * Helper method to create a basic autocompleter for codemirror to match our
 * current ace autocompleter behavior:
 *
 * - Ignore empty prefixes and comments
 * - Autocomplete only when cursor is near something that passes as identifier
 * - Have autocopmletion logic split depending on whether or not autocompletion
 *   happens inside a string token or not
 */
function createAceCompatAutocompleter(completers) {
    return function aceCompatAutocompleter(context) {
        const prefix = context.matchBefore(exports.ID_REGEX);
        const token = (0, utils_1.resolveTokenAtCursor)(context);
        if (!prefix || prefix.text === '') {
            return null;
        }
        if (['BlockComment', 'LineComment'].includes(token.type.name)) {
            return null;
        }
        const opts = { context, prefix, token };
        if (token.type.name === 'String' && completers.String) {
            return completers.String(opts) ?? null;
        }
        return completers.IdentifierLike?.(opts) ?? null;
    };
}
exports.createAceCompatAutocompleter = createAceCompatAutocompleter;
//# sourceMappingURL=ace-compat-autocompleter.js.map